local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local GetSpellLink = GetSpellLink
local GetTime = GetTime
local GetUnitName = GetUnitName
local SendChatMessage = SendChatMessage
local UnitExists = UnitExists
local UnitInRange = UnitInRange
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local string = string

a.Rotations = {}
c.RegisterForEvents(a)
s.Spam[AddonName] = function()
	c.Flash(a)
end

local oorTime = {}
local oorCount = {}
local announceTimes = {}

local function healingNeeded()
	for unit in c.GetGroupMembers() do
		if s.HealthPercent(unit) < 90 and UnitInRange(unit) then
			return true
		end
	end
end

local function getMessage(name)
	local message = a.GetConfig("announcement")
	if name then
		message = string.gsub(message, "<player>", name)
		message = string.gsub(message, "<Player>", name)
		message = string.gsub(message, "<is/are>", "is")
	else
		message = string.gsub(message, "<player>", "you")
		message = string.gsub(message, "<Player>", "You")
		message = string.gsub(message, "<is/are>", "are")
	end
	message = string.gsub(
		message, "<Atonement>", GetSpellLink(c.GetID("Atonement")))
	return message
end

local function announceFor(name)
	local now = GetTime()
	local last = announceTimes[name]
	if last ~= nil and now - last < 6 then
		return
	end
	
	if not a.GetConfig("to_raid_off") then
		if s.InRaid() then
			SendChatMessage(getMessage(name), "RAID")
		elseif s.InParty() then
			SendChatMessage(getMessage(name), "PARTY")
		end
	end
	if a.GetConfig("to_say") then
		SendChatMessage(getMessage(name))
	end
	if a.GetConfig("to_whisper") then
		SendChatMessage(getMessage(), "WHISPER", nil, name)
	end
	announceTimes[name] = now
end

local function announceUnder(percentage)
	for unit in c.GetGroupMembers() do
		local p = s.HealthPercent(unit)
		local name = GetUnitName(unit, true)
		if p / 100 < percentage
			and UnitInRange(unit)
			and not UnitIsDeadOrGhost(unit) then
			
			c.Debug("Announce", name, p, "<", percentage)
			local count = oorCount[name]
			local now = GetTime()
			if count == nil then
				oorCount[name] = 1
			else
				oorCount[name] = count + 1
			end
			if count > 2 and now - oorTime[name] < 3 then
				announceFor(name)
			end
			oorTime[name] = now
		else
			oorCount[name] = nil
		end
	end
end

a.Rotations.Atonement = {
	Spec = 1,
	
	FlashInCombat = function()
		c.FlashAll("Shadowfiend")
		
		local mana = s.PowerPercent("player")
		if healingNeeded() then
			if mana < a.GetConfig("conserve_percent") then
				c.FlashAll("Archangel")
			end
			c.PriorityFlash("Power Infusion", "Holy Fire", "Smite")
			return
		end
		
		if mana < a.GetConfig("heal_only_percent") then
			return
		end
		
		if mana < a.GetConfig("conserve_percent") then
			c.PriorityFlash(
				"Power Infusion", 
				"Archangel",
				"Holy Fire",
				"Pennance",
				"Smite")
		else
			c.PriorityFlash(
				"Power Infusion", 
				"Holy Fire",
				"Devouring Plague",
				"Pennance at 4",
				"Shadow Word: Pain",
				"Pennance at 2",
				"Mind Blast",
				"Pennance",
				"Smite")
		end
	end,
	
	FlashAlways = function()
		c.FlashAll(
			"Power Word: Fortitude",
			"Shadow Protection",
			"Inner Fire")
		if c.GetBuffDuration("Evangelism") < 5 then
			c.FlashAll("Archangel")
		end
	end,
	
	SpellHeal = function(spellID, target, amount)
		if spellID == c.GetID("Atonement") and UnitExists(target) then
			announceUnder((s.Health(target) - amount) / s.MaxHealth(target))
		end
	end,
	
	LeftCombat = function()
		oorTime = {}
		oorCount = {}
		announceTimes = {}
		c.Debug("Event", "Clearing announcement history")
	end
}
